<!DOCTYPE html>
<html>
<head>
<meta charset=utf-8>
<title>Redirecting...</title>
<link rel=canonical href="{{ redirect_url }}">
<meta http-equiv=refresh content="0; url={{ redirect_url }}">
<h1>Redirecting...</h1>
<a href="{{ redirect_url }}">Click here if you are not redirected.</a>
<script>location='{{ redirect_url }}'</script>
</body>
</html>
